@@ -1,6 +1,5 @@
 import typing
 
-import six as _six
 from flyteidl.admin import common_pb2 as _common_pb2
 from flyteidl.admin import execution_pb2 as _execution_pb2
 from flyteidl.admin import launch_plan_pb2 as _launch_plan_pb2
@@ -114,7 +113,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
         return (
             [_common.NamedEntityIdentifier.from_flyte_idl(identifier_pb) for identifier_pb in identifier_list.entities],
-            _six.text_type(identifier_list.token),
+            str(identifier_list.token),
         )
 
     def list_tasks_paginated(self, identifier, limit=100, token=None, filters=None, sort_by=None):
@@ -159,7 +158,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             pb.id.resource_type = _identifier.ResourceType.TASK
         return (
             [_task.Task.from_flyte_idl(task_pb2) for task_pb2 in task_list.tasks],
-            _six.text_type(task_list.token),
+            str(task_list.token),
         )
 
     def get_task(self, id):
@@ -243,7 +242,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
         return (
             [_common.NamedEntityIdentifier.from_flyte_idl(identifier_pb) for identifier_pb in identifier_list.entities],
-            _six.text_type(identifier_list.token),
+            str(identifier_list.token),
         )
 
     def list_workflows_paginated(self, identifier, limit=100, token=None, filters=None, sort_by=None):
@@ -288,12 +287,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             pb.id.resource_type = _identifier.ResourceType.WORKFLOW
         return (
             [_workflow.Workflow.from_flyte_idl(wf_pb2) for wf_pb2 in wf_list.workflows],
-            _six.text_type(wf_list.token),
+            str(wf_list.token),
         )
 
     def get_workflow(self, id):
         """
-        This returns a single task for a given ID.
+        This returns a single workflow for a given ID.
 
         :param flytekit.models.core.identifier.Identifier id: The ID representing a given task.
         :raises: TODO
@@ -398,7 +397,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
         return (
             [_common.NamedEntityIdentifier.from_flyte_idl(identifier_pb) for identifier_pb in identifier_list.entities],
-            _six.text_type(identifier_list.token),
+            str(identifier_list.token),
         )
 
     def list_launch_plans_paginated(self, identifier, limit=100, token=None, filters=None, sort_by=None):
@@ -443,7 +442,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             pb.id.resource_type = _identifier.ResourceType.LAUNCH_PLAN
         return (
             [_launch_plan.LaunchPlan.from_flyte_idl(pb) for pb in lp_list.launch_plans],
-            _six.text_type(lp_list.token),
+            str(lp_list.token),
         )
 
     def list_active_launch_plans_paginated(self, project, domain, limit=100, token=None, sort_by=None):
@@ -487,7 +486,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             pb.id.resource_type = _identifier.ResourceType.LAUNCH_PLAN
         return (
             [_launch_plan.LaunchPlan.from_flyte_idl(pb) for pb in lp_list.launch_plans],
-            _six.text_type(lp_list.token),
+            str(lp_list.token),
         )
 
     def update_launch_plan(self, id, state):
@@ -558,6 +557,19 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
             .id
         )
 
+    def recover_execution(self, id, name: str = None):
+        """
+        Recreates a previously-run workflow execution that will only start executing from the last known failure point.
+        :param flytekit.common.core.identifier.WorkflowExecutionIdentifier id:
+        :param name str: Optional name to assign to the newly created execution.
+        :rtype: flytekit.models.core.identifier.WorkflowExecutionIdentifier
+        """
+        return _identifier.WorkflowExecutionIdentifier.from_flyte_idl(
+            super(SynchronousFlyteClient, self)
+            .recover_execution(_execution_pb2.ExecutionRecoverRequest(id=id.to_flyte_idl(), name=name))
+            .id
+        )
+
     def get_execution(self, id):
         """
         :param flytekit.common.core.identifier.WorkflowExecutionIdentifier id:
@@ -621,7 +633,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
         return (
             [_execution.Execution.from_flyte_idl(pb) for pb in exec_list.executions],
-            _six.text_type(exec_list.token),
+            str(exec_list.token),
         )
 
     def terminate_execution(self, id, cause):
@@ -710,7 +722,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
         return (
             [_node_execution.NodeExecution.from_flyte_idl(e) for e in exec_list.node_executions],
-            _six.text_type(exec_list.token),
+            str(exec_list.token),
         )
 
     def list_node_executions_for_task_paginated(
@@ -743,7 +755,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
         return (
             [_node_execution.NodeExecution.from_flyte_idl(e) for e in exec_list.node_executions],
-            _six.text_type(exec_list.token),
+            str(exec_list.token),
         )
 
     ####################################################################################################################
@@ -805,7 +817,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
         return (
             [_task_execution.TaskExecution.from_flyte_idl(e) for e in exec_list.task_executions],
-            _six.text_type(exec_list.token),
+            str(exec_list.token),
         )
 
     ####################################################################################################################
@@ -870,7 +882,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
         return (
             [_project.Project.from_flyte_idl(pb) for pb in projects.projects],
-            _six.text_type(projects.token),
+            str(projects.token),
         )
 
     ####################################################################################################################
