@@ -142,7 +142,7 @@ def _handle_rpc_error(retry=False):
                             if (retry is False) or i == (max_retries - 1):
                                 raise
                             else:
-                                # Retry: Start with 200ms wait-time and exponentially back-off upto 1 second.
+                                # Retry: Start with 200ms wait-time and exponentially back-off up to 1 second.
                                 wait_time = min(200 * (2 ** i), max_wait_time)
                                 cli_logger.error(f"Non-auth RPC error {e}, sleeping {wait_time}ms and retrying")
                                 time.sleep(wait_time / 1000)
@@ -475,7 +475,7 @@ class RawSynchronousFlyteClient(object):
     @_handle_rpc_error(retry=True)
     def list_launch_plans_paginated(self, resource_list_request):
         """
-        Lists Launch Plans for a given Identifer (project, domain, name)
+        Lists Launch Plans for a given Identifier (project, domain, name)
 
         :param: flyteidl.admin.common_pb2.ResourceListRequest resource_list_request:
         :rtype: flyteidl.admin.launch_plan_pb2.LaunchPlanList
@@ -521,6 +521,15 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.CreateExecution(create_execution_request, metadata=self._metadata)
 
+    @_handle_rpc_error()
+    def recover_execution(self, recover_execution_request):
+        """
+        This will recreate an execution with the same spec as the one belonging to the given execution identifier.
+        :param flyteidl.admin.execution_pb2.ExecutionRecoverRequest recover_execution_request:
+        :rtype: flyteidl.admin.execution_pb2.ExecutionRecoverResponse
+        """
+        return self._stub.RecoverExecution(recover_execution_request, metadata=self._metadata)
+
     @_handle_rpc_error(retry=True)
     def get_execution(self, get_object_request):
         """
@@ -670,7 +679,7 @@ class RawSynchronousFlyteClient(object):
         :param flyteidl.admin.project_pb2.Project project:
         :rtype: flyteidl.admin.project_pb2.ProjectUpdateResponse
         """
-        return self._stub.UpdateProject(project)
+        return self._stub.UpdateProject(project, metadata=self._metadata)
 
     ####################################################################################################################
     #
