@@ -1,15 +1,10 @@
-from __future__ import absolute_import
-
-import six as _six
-
-
 class _FlyteCodedExceptionMetaclass(type):
     @property
     def error_code(cls):
         return cls._ERROR_CODE
 
 
-class FlyteException(_six.with_metaclass(_FlyteCodedExceptionMetaclass, Exception)):
+class FlyteException(Exception, metaclass=_FlyteCodedExceptionMetaclass):
     _ERROR_CODE = "UnknownFlyteException"
 
 
