@@ -1,7 +1,3 @@
-from __future__ import absolute_import
-
-import six as _six
-
 from flytekit.common import constants as _constants
 from flytekit.common import sdk_bases as _sdk_bases
 from flytekit.common.exceptions import user as _user_exceptions
@@ -10,7 +6,7 @@ from flytekit.models import interface as _interface_models
 from flytekit.models import types as _type_models
 
 
-class Input(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _interface_models.Parameter)):
+class Input(_interface_models.Parameter, metaclass=_sdk_bases.ExtendedSdkType):
     def __init__(self, name, sdk_type, help=None, **kwargs):
         """
         :param Text name:
@@ -107,23 +103,29 @@ class Input(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _interface_models.Pa
     def promote_from_model(cls, model):
         """
         :param flytekit.models.interface.Parameter model:
-        :rtype: Parameter
+        :rtype: Input
         """
         sdk_type = _type_helpers.get_sdk_type_from_literal_type(model.var.type)
 
         if model.default is not None:
             default_value = sdk_type.from_flyte_idl(model.default.to_flyte_idl()).to_python_std()
-            return cls("", sdk_type, help=model.var.description, required=False, default=default_value,)
+            return cls(
+                "",
+                sdk_type,
+                help=model.var.description,
+                required=False,
+                default=default_value,
+            )
         else:
             return cls("", sdk_type, help=model.var.description, required=True)
 
 
-class NodeOutput(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _type_models.OutputReference)):
+class NodeOutput(_type_models.OutputReference, metaclass=_sdk_bases.ExtendedSdkType):
     def __init__(self, sdk_node, sdk_type, var):
         """
-        :param flytekit.common.nodes.SdkNode sdk_node:
-        :param flytekit.common.types.FlyteSdkType sdk_type:
-        :param Text var:
+        :param sdk_node:
+        :param sdk_type: deprecated in mypy flytekit.
+        :param var:
         """
         self._node = sdk_node
         self._type = sdk_type
@@ -163,4 +165,5 @@ class NodeOutput(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _type_models.Ou
         return self._type
 
     def __repr__(self):
-        return "NodeOutput({}:{})".format(self.sdk_node, self.var)
+        s = f"NodeOutput({self.sdk_node if self.sdk_node.id is not None else None}:{self.var})"
+        return s
