@@ -1,16 +1,21 @@
-from __future__ import absolute_import
+from __future__ import annotations
 
-try:
-    from inspect import getfullargspec as _getargspec
-except ImportError:
-    from inspect import getargspec as _getargspec
+import copy as _copy
+import enum
+import logging as _logging
+import os
+import pathlib
+import typing
+from dataclasses import dataclass
+from datetime import datetime
+from inspect import getfullargspec as _getargspec
 
 import six as _six
 
-from flytekit import __version__
 from flytekit.common import constants as _constants
 from flytekit.common import interface as _interface
 from flytekit.common import sdk_bases as _sdk_bases
+from flytekit.common import utils as _common_utils
 from flytekit.common.core.identifier import WorkflowExecutionIdentifier
 from flytekit.common.exceptions import scopes as _exception_scopes
 from flytekit.common.exceptions import user as _user_exceptions
@@ -20,60 +25,174 @@ from flytekit.common.types import helpers as _type_helpers
 from flytekit.configuration import internal as _internal_config
 from flytekit.configuration import resources as _resource_config
 from flytekit.configuration import sdk as _sdk_config
+from flytekit.configuration import secrets
 from flytekit.engines import loader as _engine_loader
+from flytekit.interfaces.stats import taggable
 from flytekit.models import literals as _literal_models
 from flytekit.models import task as _task_models
 
 
+class SecretsManager(object):
+    """
+    This provides a secrets resolution logic at runtime.
+    The resolution order is
+      - Try env var first. The env var should have the configuration.SECRETS_ENV_PREFIX. The env var will be all upper
+         cased
+      - If not then try the file where the name matches lower case
+        ``configuration.SECRETS_DEFAULT_DIR/<group>/configuration.SECRETS_FILE_PREFIX<key>``
+
+    All configuration values can always be overridden by injecting an environment variable
+    """
+
+    def __init__(self):
+        self._base_dir = str(secrets.SECRETS_DEFAULT_DIR.get()).strip()
+        self._file_prefix = str(secrets.SECRETS_FILE_PREFIX.get()).strip()
+        self._env_prefix = str(secrets.SECRETS_ENV_PREFIX.get()).strip()
+
+    def get(self, group: str, key: str) -> str:
+        """
+        Retrieves a secret using the resolution order -> Env followed by file. If not found raises a ValueError
+        """
+        self.check_group_key(group, key)
+        env_var = self.get_secrets_env_var(group, key)
+        fpath = self.get_secrets_file(group, key)
+        v = os.environ.get(env_var)
+        if v is not None:
+            return v
+        if os.path.exists(fpath):
+            with open(fpath, "r") as f:
+                return f.read().strip()
+        raise ValueError(
+            f"Unable to find secret for key {key} in group {group} " f"in Env Var:{env_var} and FilePath: {fpath}"
+        )
+
+    def get_secrets_env_var(self, group: str, key: str) -> str:
+        """
+        Returns a string that matches the ENV Variable to look for the secrets
+        """
+        self.check_group_key(group, key)
+        return f"{self._env_prefix}{group.upper()}_{key.upper()}"
+
+    def get_secrets_file(self, group: str, key: str) -> str:
+        """
+        Returns a path that matches the file to look for the secrets
+        """
+        self.check_group_key(group, key)
+        return os.path.join(self._base_dir, group.lower(), f"{self._file_prefix}{key.lower()}")
+
+    @staticmethod
+    def check_group_key(group: str, key: str):
+        if group is None or group == "":
+            raise ValueError("secrets group is a mandatory field.")
+        if key is None or key == "":
+            raise ValueError("secrets key is a mandatory field.")
+
+
+# TODO: Clean up working dir name
 class ExecutionParameters(object):
-
     """
-    This is the parameter object that will be provided as the first parameter for every execution of any @*_task
-    decorated function.
+    This is a run-time user-centric context object that is accessible to every @task method. It can be accessed using
+
+    .. code-block:: python
+
+        flytekit.current_context()
+
+    This object provides the following
+    * a statsd handler
+    * a logging handler
+    * the execution ID as an :py:class:`flytekit.models.core.identifier.WorkflowExecutionIdentifier` object
+    * a working directory for the user to write arbitrary files to
+
+    Please do not confuse this object with the :py:class:`flytekit.FlyteContext` object.
     """
 
-    def __init__(self, execution_date, tmp_dir, stats, execution_id, logging):
+    @dataclass(init=False)
+    class Builder(object):
+        stats: taggable.TaggableStats
+        execution_date: datetime
+        logging: _logging
+        execution_id: str
+        attrs: typing.Dict[str, typing.Any]
+        working_dir: typing.Union[os.PathLike, _common_utils.AutoDeletingTempDir]
+
+        def __init__(self, current: typing.Optional[ExecutionParameters] = None):
+            self.stats = current.stats if current else None
+            self.execution_date = current.execution_date if current else None
+            self.working_dir = current.working_directory if current else None
+            self.execution_id = current.execution_id if current else None
+            self.logging = current.logging if current else None
+            self.attrs = current._attrs if current else {}
+
+        def add_attr(self, key: str, v: typing.Any) -> ExecutionParameters.Builder:
+            self.attrs[key] = v
+            return self
+
+        def build(self) -> ExecutionParameters:
+            if not isinstance(self.working_dir, _common_utils.AutoDeletingTempDir):
+                pathlib.Path(self.working_dir).mkdir(parents=True, exist_ok=True)
+            return ExecutionParameters(
+                execution_date=self.execution_date,
+                stats=self.stats,
+                tmp_dir=self.working_dir,
+                execution_id=self.execution_id,
+                logging=self.logging,
+                **self.attrs,
+            )
+
+    @staticmethod
+    def new_builder(current: ExecutionParameters = None) -> Builder:
+        return ExecutionParameters.Builder(current=current)
+
+    def builder(self) -> Builder:
+        return ExecutionParameters.Builder(current=self)
+
+    def __init__(self, execution_date, tmp_dir, stats, execution_id, logging, **kwargs):
+        """
+        Args:
+            execution_date: Date when the execution is running
+            tmp_dir: temporary directory for the execution
+            stats: handle to emit stats
+            execution_id: Identifier for the xecution
+            logging: handle to logging
+        """
         self._stats = stats
         self._execution_date = execution_date
         self._working_directory = tmp_dir
         self._execution_id = execution_id
         self._logging = logging
+        # AutoDeletingTempDir's should be used with a with block, which creates upon entry
+        self._attrs = kwargs
+        # It is safe to recreate the Secrets Manager
+        self._secrets_manager = SecretsManager()
 
     @property
-    def stats(self):
+    def stats(self) -> taggable.TaggableStats:
         """
         A handle to a special statsd object that provides usefully tagged stats.
-
         TODO: Usage examples and better comments
-
-        :rtype: flytekit.interfaces.stats.taggable.TaggableStats
         """
         return self._stats
 
     @property
-    def logging(self):
+    def logging(self) -> _logging:
         """
         A handle to a useful logging object.
-
         TODO: Usage examples
-
-        :rtype: logging
         """
         return self._logging
 
     @property
-    def working_directory(self):
+    def working_directory(self) -> _common_utils.AutoDeletingTempDir:
         """
         A handle to a special working directory for easily producing temporary files.
 
         TODO: Usage examples
-
-        :rtype: flytekit.common.utils.AutoDeletingTempDir
+        TODO: This does not always return a AutoDeletingTempDir
         """
         return self._working_directory
 
     @property
-    def execution_date(self):
+    def execution_date(self) -> datetime:
         """
         This is a datetime representing the time at which a workflow was started.  This is consistent across all tasks
         executed in a workflow or sub-workflow.
@@ -82,13 +201,11 @@ class ExecutionParameters(object):
 
             Do NOT use this execution_date to drive any production logic.  It might be useful as a tag for data to help
             in debugging.
-
-        :rtype: datetime.datetime
         """
         return self._execution_date
 
     @property
-    def execution_id(self):
+    def execution_id(self) -> str:
         """
         This is the identifier of the workflow execution within the underlying engine.  It will be consistent across all
         task executions in a workflow or sub-workflow execution.
@@ -97,15 +214,47 @@ class ExecutionParameters(object):
 
             Do NOT use this execution_id to drive any production logic.  This execution ID should only be used as a tag
             on output data to link back to the workflow run that created it.
-
-        :rtype: Text
         """
         return self._execution_id
 
+    @property
+    def secrets(self) -> SecretsManager:
+        return self._secrets_manager
+
+    def __getattr__(self, attr_name: str) -> typing.Any:
+        """
+        This houses certain task specific context. For example in Spark, it houses the SparkSession, etc
+        """
+        attr_name = attr_name.upper()
+        if self._attrs and attr_name in self._attrs:
+            return self._attrs[attr_name]
+        raise AssertionError(f"{attr_name} not available as a parameter in Flyte context - are you in right task-type?")
+
+    def has_attr(self, attr_name: str) -> bool:
+        attr_name = attr_name.upper()
+        if self._attrs and attr_name in self._attrs:
+            return True
+        return False
+
+    def get(self, key: str) -> typing.Any:
+        """
+        Returns task specific context if present else raise an error. The returned context will match the key
+        """
+        return self.__getattr__(attr_name=key)
+
+
+class SdkRunnableContainer(_task_models.Container, metaclass=_sdk_bases.ExtendedSdkType):
+    """
+    This is not necessarily a local-only Container object. So long as configuration is present, you can use this object
+    """
 
-class SdkRunnableContainer(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _task_models.Container)):
     def __init__(
-        self, command, args, resources, env, config,
+        self,
+        command,
+        args,
+        resources,
+        env,
+        config,
     ):
         super(SdkRunnableContainer, self).__init__("", command, args, resources, env or {}, config)
 
@@ -142,8 +291,65 @@ class SdkRunnableContainer(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _task
         )
         return env
 
+    @classmethod
+    def get_resources(
+        cls,
+        storage_request=None,
+        cpu_request=None,
+        gpu_request=None,
+        memory_request=None,
+        storage_limit=None,
+        cpu_limit=None,
+        gpu_limit=None,
+        memory_limit=None,
+    ):
+        """
+        :param Text storage_request:
+        :param Text cpu_request:
+        :param Text gpu_request:
+        :param Text memory_request:
+        :param Text storage_limit:
+        :param Text cpu_limit:
+        :param Text gpu_limit:
+        :param Text memory_limit:
+        """
+        requests = []
+        if storage_request:
+            requests.append(
+                _task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.STORAGE, storage_request)
+            )
+        if cpu_request:
+            requests.append(_task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.CPU, cpu_request))
+        if gpu_request:
+            requests.append(_task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.GPU, gpu_request))
+        if memory_request:
+            requests.append(
+                _task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.MEMORY, memory_request)
+            )
 
-class SdkRunnableTask(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _base_task.SdkTask)):
+        limits = []
+        if storage_limit:
+            limits.append(
+                _task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.STORAGE, storage_limit)
+            )
+        if cpu_limit:
+            limits.append(_task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.CPU, cpu_limit))
+        if gpu_limit:
+            limits.append(_task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.GPU, gpu_limit))
+        if memory_limit:
+            limits.append(
+                _task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.MEMORY, memory_limit)
+            )
+
+        return _task_models.Resources(limits=limits, requests=requests)
+
+
+class SdkRunnableTaskStyle(enum.Enum):
+    V0 = 0
+    V1 = 1
+
+
+class SdkRunnableTask(_base_task.SdkTask, metaclass=_sdk_bases.ExtendedSdkType):
     """
     This class includes the additional logic for building a task that executes in Python code.  It has even more
     validation checks to ensure proper behavior than it's superclasses.
@@ -193,14 +399,18 @@ class SdkRunnableTask(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _base_task
         :param dict[Text, Text] environment:
         :param dict[Text, T] custom:
         """
-        self._task_function = task_function
+        # Circular dependency
+        from flytekit import __version__
 
+        self._task_function = task_function
         super(SdkRunnableTask, self).__init__(
             task_type,
             _task_models.TaskMetadata(
                 discoverable,
                 _task_models.RuntimeMetadata(
-                    _task_models.RuntimeMetadata.RuntimeType.FLYTE_SDK, __version__, "python",
+                    _task_models.RuntimeMetadata.RuntimeType.FLYTE_SDK,
+                    __version__,
+                    "python",
                 ),
                 timeout,
                 _literal_models.RetryStrategy(retries),
@@ -208,6 +418,7 @@ class SdkRunnableTask(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _base_task
                 discovery_version,
                 deprecated,
             ),
+            # TODO: If we end up using SdkRunnableTask for the new code, make sure this is set correctly.
             _interface.TypedInterface({}, {}),
             custom,
             container=self._get_container_definition(
@@ -223,6 +434,10 @@ class SdkRunnableTask(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _base_task
             ),
         )
         self.id._name = "{}.{}".format(self.task_module, self.task_function_name)
+        self._has_fast_registered = False
+
+        # TODO: Remove this in the future, I don't think we'll be using this.
+        self._task_style = SdkRunnableTaskStyle.V0
 
     _banned_inputs = {}
     _banned_outputs = {}
@@ -243,6 +458,10 @@ class SdkRunnableTask(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _base_task
         # TODO: If the task exists in this container, we should be able to retrieve it.
         raise _user_exceptions.FlyteAssertion("Cannot promote a base object to a runnable task.")
 
+    @property
+    def task_style(self):
+        return self._task_style
+
     @property
     def task_function(self):
         return self._task_function
@@ -329,18 +548,18 @@ class SdkRunnableTask(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _base_task
             working directory (with the names provided), which will in turn allow Flyte Propeller to push along the
             workflow.  Where as local engine will merely feed the outputs directly into the next node.
         """
-
-        return _exception_scopes.user_entry_point(self.task_function)(
-            ExecutionParameters(
-                execution_date=context.execution_date,
-                # TODO: it might be better to consider passing the full struct
-                execution_id=_six.text_type(WorkflowExecutionIdentifier.promote_from_model(context.execution_id)),
-                stats=context.stats,
-                logging=context.logging,
-                tmp_dir=context.working_directory,
-            ),
-            **inputs
-        )
+        if self.task_style == SdkRunnableTaskStyle.V0:
+            return _exception_scopes.user_entry_point(self.task_function)(
+                ExecutionParameters(
+                    execution_date=context.execution_date,
+                    # TODO: it might be better to consider passing the full struct
+                    execution_id=_six.text_type(WorkflowExecutionIdentifier.promote_from_model(context.execution_id)),
+                    stats=context.stats,
+                    logging=context.logging,
+                    tmp_dir=context.working_directory,
+                ),
+                **inputs,
+            )
 
     @_exception_scopes.system_entry_point
     def execute(self, context, inputs):
@@ -355,22 +574,67 @@ class SdkRunnableTask(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _base_task
             workflow.  Where as local engine will merely feed the outputs directly into the next node.
         """
         inputs_dict = _type_helpers.unpack_literal_map_to_sdk_python_std(
-            inputs,
-            {k: _type_helpers.get_sdk_type_from_literal_type(v.type) for k, v in _six.iteritems(self.interface.inputs)},
+            inputs, {k: _type_helpers.get_sdk_type_from_literal_type(v.type) for k, v in self.interface.inputs.items()}
         )
         outputs_dict = {
             name: _task_output.OutputReference(_type_helpers.get_sdk_type_from_literal_type(variable.type))
             for name, variable in _six.iteritems(self.interface.outputs)
         }
-        inputs_dict.update(outputs_dict)
 
-        self._execute_user_code(context, inputs_dict)
+        # Old style - V0: If annotations are used to define outputs, do not append outputs to the inputs dict
+        if not self.task_function.__annotations__ or "return" not in self.task_function.__annotations__:
+            inputs_dict.update(outputs_dict)
+            self._execute_user_code(context, inputs_dict)
+            return {
+                _constants.OUTPUT_FILE_NAME: _literal_models.LiteralMap(
+                    literals={k: v.sdk_value for k, v in _six.iteritems(outputs_dict)}
+                )
+            }
 
-        return {
-            _constants.OUTPUT_FILE_NAME: _literal_models.LiteralMap(
-                literals={k: v.sdk_value for k, v in _six.iteritems(outputs_dict)}
-            )
-        }
+    @_exception_scopes.system_entry_point
+    def fast_register(self, project, domain, name, digest, additional_distribution, dest_dir) -> str:
+        """
+        The fast register call essentially hijacks the task container commandline.
+        Say an existing task container definition had a commandline like so:
+            flyte_venv pyflyte-execute --task-module app.workflows.my_workflow --task-name my_task
+
+        The fast register command introduces a wrapper call to fast-execute the original commandline like so:
+            flyte_venv pyflyte-fast-execute --additional-distribution s3://my-s3-bucket/foo/bar/12345.tar.gz --
+                flyte_venv pyflyte-execute --task-module app.workflows.my_workflow --task-name my_task
+
+        At execution time pyflyte-fast-execute will ensure the additional distribution (i.e. the fast-registered code)
+        exists before calling the original task commandline.
+
+        :param Text project: The project in which to register this task.
+        :param Text domain: The domain in which to register this task.
+        :param Text name: The name to give this task.
+        :param Text digest: The version in which to register this task.
+        :param Text additional_distribution: User-specified location for remote source code distribution.
+        :param Text The optional location for where to install the additional distribution at runtime
+        :rtype: Text: Registered identifier.
+        """
+
+        original_container = self.container
+        container = _copy.deepcopy(original_container)
+        args = ["pyflyte-fast-execute", "--additional-distribution", additional_distribution]
+        if dest_dir:
+            args += ["--dest-dir", dest_dir]
+        args += ["--"] + container.args
+        container._args = args
+        self._container = container
+
+        try:
+            registered_id = self.register(project, domain, name, digest)
+        except Exception:
+            self._container = original_container
+            raise
+        self._has_fast_registered = True
+        self._container = original_container
+        return str(registered_id)
+
+    @property
+    def has_fast_registered(self) -> bool:
+        return self._has_fast_registered
 
     def _get_container_definition(
         self,
@@ -407,33 +671,9 @@ class SdkRunnableTask(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _base_task
         memory_limit = memory_limit or _resource_config.DEFAULT_MEMORY_LIMIT.get()
         memory_request = memory_request or _resource_config.DEFAULT_MEMORY_REQUEST.get()
 
-        requests = []
-        if storage_request:
-            requests.append(
-                _task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.STORAGE, storage_request)
-            )
-        if cpu_request:
-            requests.append(_task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.CPU, cpu_request))
-        if gpu_request:
-            requests.append(_task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.GPU, gpu_request))
-        if memory_request:
-            requests.append(
-                _task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.MEMORY, memory_request)
-            )
-
-        limits = []
-        if storage_limit:
-            limits.append(
-                _task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.STORAGE, storage_limit)
-            )
-        if cpu_limit:
-            limits.append(_task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.CPU, cpu_limit))
-        if gpu_limit:
-            limits.append(_task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.GPU, gpu_limit))
-        if memory_limit:
-            limits.append(
-                _task_models.Resources.ResourceEntry(_task_models.Resources.ResourceName.MEMORY, memory_limit)
-            )
+        resources = SdkRunnableContainer.get_resources(
+            storage_request, cpu_request, gpu_request, memory_request, storage_limit, cpu_limit, gpu_limit, memory_limit
+        )
 
         return (cls or SdkRunnableContainer)(
             command=[],
@@ -450,7 +690,7 @@ class SdkRunnableTask(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _base_task
                 "--raw-output-data-prefix",
                 "{{.rawOutputDataPrefix}}",
             ],
-            resources=_task_models.Resources(limits=limits, requests=requests),
+            resources=resources,
             env=environment,
             config={},
         )
