@@ -1,5 +1,5 @@
 from collections import OrderedDict
-from typing import Callable, List, Optional, Union
+from typing import Callable, Dict, List, Optional, Tuple, Union
 
 from flytekit.common import constants as _common_constants
 from flytekit.common.utils import _dnsify
@@ -9,7 +9,7 @@ from flytekit.core.context_manager import SerializationSettings
 from flytekit.core.launch_plan import LaunchPlan, ReferenceLaunchPlan
 from flytekit.core.node import Node
 from flytekit.core.python_auto_container import PythonAutoContainerTask
-from flytekit.core.reference_entity import ReferenceEntity
+from flytekit.core.reference_entity import ReferenceEntity, ReferenceSpec, ReferenceTemplate
 from flytekit.core.task import ReferenceTask
 from flytekit.core.workflow import ReferenceWorkflow, WorkflowBase
 from flytekit.models import common as _common_models
@@ -138,15 +138,17 @@ def get_serializable_workflow(
     sub_wfs = []
     for n in entity.nodes:
         if isinstance(n.flyte_entity, WorkflowBase):
-            if isinstance(n.flyte_entity, ReferenceEntity):
+            # We are currently not supporting reference workflows since these will
+            # require a network call to flyteadmin to populate the WorkflowTemplate
+            # object
+            if isinstance(n.flyte_entity, ReferenceWorkflow):
                 raise Exception(
-                    f"Sorry, reference subworkflows do not work right now, please use the launch plan instead for the "
-                    f"subworkflow you're trying to invoke. Node: {n}"
+                    "Reference sub-workflows are currently unsupported. Use reference launch plans instead."
                 )
             sub_wf_spec = get_serializable(entity_mapping, settings, n.flyte_entity)
             if not isinstance(sub_wf_spec, admin_workflow_models.WorkflowSpec):
-                raise Exception(
-                    f"Serialized form of a workflow should be an admin.WorkflowSpec but {type(sub_wf_spec)} found instead"
+                raise TypeError(
+                    f"Unexpected type for serialized form of workflow. Expected {admin_workflow_models.WorkflowSpec}, but got {type(sub_wf_spec)}"
                 )
             sub_wfs.append(sub_wf_spec.template)
             sub_wfs.extend(sub_wf_spec.sub_workflows)
@@ -245,11 +247,8 @@ def get_serializable_node(
 
     # Reference entities also inherit from the classes in the second if statement so address them first.
     if isinstance(entity.flyte_entity, ReferenceEntity):
-        # This is a throw away call.
-        # See the comment in compile_into_workflow in python_function_task. This is just used to place a None value
-        # in the entity_mapping.
-        get_serializable(entity_mapping, settings, entity.flyte_entity)
-        ref = entity.flyte_entity
+        ref_spec = get_serializable(entity_mapping, settings, entity.flyte_entity)
+        ref_template = ref_spec.template
         node_model = workflow_model.Node(
             id=_dnsify(entity.id),
             metadata=entity.metadata,
@@ -257,14 +256,16 @@ def get_serializable_node(
             upstream_node_ids=[n.id for n in upstream_sdk_nodes],
             output_aliases=[],
         )
-        if ref.reference.resource_type == _identifier_model.ResourceType.TASK:
-            node_model._task_node = workflow_model.TaskNode(reference_id=ref.id)
-        elif ref.reference.resource_type == _identifier_model.ResourceType.WORKFLOW:
-            node_model._workflow_node = workflow_model.WorkflowNode(sub_workflow_ref=ref.id)
-        elif ref.reference.resource_type == _identifier_model.ResourceType.LAUNCH_PLAN:
-            node_model._workflow_node = workflow_model.WorkflowNode(launchplan_ref=ref.id)
+        if ref_template.resource_type == _identifier_model.ResourceType.TASK:
+            node_model._task_node = workflow_model.TaskNode(reference_id=ref_template.id)
+        elif ref_template.resource_type == _identifier_model.ResourceType.WORKFLOW:
+            node_model._workflow_node = workflow_model.WorkflowNode(sub_workflow_ref=ref_template.id)
+        elif ref_template.resource_type == _identifier_model.ResourceType.LAUNCH_PLAN:
+            node_model._workflow_node = workflow_model.WorkflowNode(launchplan_ref=ref_template.id)
         else:
-            raise Exception(f"Unexpected reference type {ref}")
+            raise Exception(
+                f"Unexpected resource type for reference entity {entity.flyte_entity}: {ref_template.resource_type}"
+            )
         return node_model
 
     if isinstance(entity.flyte_entity, PythonTask):
@@ -328,7 +329,7 @@ def get_serializable_branch_node(
     # We have to iterate through the blocks to convert the nodes from the internal Node type to the Node model type.
     # This was done to avoid having to create our own IfElseBlock object (i.e. condition.py just uses the model
     # directly) even though the node there is of the wrong type (our type instead of the model type).
-    # TODO this should be cleaned up instead of mutation, we probaby should just create a new object
+    # TODO this should be cleaned up instead of mutation, we probably should just create a new object
     first = to_serializable_case(entity_mapping, settings, entity._ifelse_block.case)
     other = to_serializable_cases(entity_mapping, settings, entity._ifelse_block.other)
     else_node_model = None
@@ -342,6 +343,13 @@ def get_serializable_branch_node(
     )
 
 
+def get_reference_spec(
+    entity_mapping: OrderedDict, settings: SerializationSettings, entity: ReferenceEntity
+) -> ReferenceSpec:
+    template = ReferenceTemplate(entity.id, entity.reference.resource_type)
+    return ReferenceSpec(template)
+
+
 def get_serializable(
     entity_mapping: OrderedDict,
     settings: SerializationSettings,
@@ -367,10 +375,7 @@ def get_serializable(
         return entity_mapping[entity]
 
     if isinstance(entity, ReferenceEntity):
-        # TODO: Create a non-registerable model class comparable to TaskSpec or WorkflowSpec to replace None as a
-        #  keystone value. The purpose is only to store something so that we can check for it when compiling
-        #  dynamic tasks. See comment in compile_into_workflow.
-        cp_entity = None
+        cp_entity = get_reference_spec(entity_mapping, settings, entity)
 
     elif isinstance(entity, PythonTask):
         cp_entity = get_serializable_task(entity_mapping, settings, entity)
@@ -392,3 +397,34 @@ def get_serializable(
     # This needs to be at the bottom not the top - i.e. dependent tasks get added before the workflow containing it
     entity_mapping[entity] = cp_entity
     return cp_entity
+
+
+def gather_dependent_entities(
+    serialized: OrderedDict,
+) -> Tuple[
+    Dict[_identifier_model.Identifier, task_models.TaskTemplate],
+    Dict[_identifier_model.Identifier, admin_workflow_models.WorkflowSpec],
+    Dict[_identifier_model.Identifier, _launch_plan_models.LaunchPlanSpec],
+]:
+    """
+    The ``get_serializable`` function above takes in an ``OrderedDict`` that helps keep track of dependent entities.
+    For example, when serializing a workflow, all its tasks are also serialized. The ordered dict will also contain
+    serialized entities that aren't as useful though, like nodes and branches. This is just a small helper function
+    that will pull out the serialzed tasks, workflows, and launch plans. This function is primarily used for testing.
+
+    :param serialized: This should be the filled in OrderedDict used in the get_serializable function above.
+    :return:
+    """
+    task_templates: Dict[_identifier_model.Identifier, task_models.TaskTemplate] = {}
+    workflow_specs: Dict[_identifier_model.Identifier, admin_workflow_models.WorkflowSpec] = {}
+    launch_plan_specs: Dict[_identifier_model.Identifier, _launch_plan_models.LaunchPlanSpec] = {}
+
+    for cp_entity in serialized.values():
+        if isinstance(cp_entity, task_models.TaskSpec):
+            task_templates[cp_entity.template.id] = cp_entity.template
+        elif isinstance(cp_entity, _launch_plan_models.LaunchPlan):
+            launch_plan_specs[cp_entity.id] = cp_entity.spec
+        elif isinstance(cp_entity, admin_workflow_models.WorkflowSpec):
+            workflow_specs[cp_entity.template.id] = cp_entity
+
+    return task_templates, workflow_specs, launch_plan_specs
