@@ -33,8 +33,8 @@ from flytekit.common.core.identifier import WorkflowExecutionIdentifier as _SdkW
 from flytekit.common.tasks.sdk_runnable import ExecutionParameters
 from flytekit.configuration import images, internal
 from flytekit.configuration import sdk as _sdk_config
+from flytekit.core.data_persistence import FileAccessProvider, default_local_file_access_provider
 from flytekit.engines.unit import mock_stats as _mock_stats
-from flytekit.interfaces.data import data_proxy as _data_proxy
 from flytekit.models.core import identifier as _identifier
 
 # TODO: resolve circular import from flytekit.core.python_auto_container import TaskResolverMixin
@@ -414,7 +414,7 @@ class FlyteContext(object):
     Please do not confuse this object with the :py:class:`flytekit.ExecutionParameters` object.
     """
 
-    file_access: Optional[_data_proxy.FileAccessProvider]
+    file_access: Optional[FileAccessProvider]
     level: int = 0
     flyte_client: Optional[friendly_client.SynchronousFlyteClient] = None
     compilation_state: Optional[CompilationState] = None
@@ -462,7 +462,7 @@ class FlyteContext(object):
     def with_new_compilation_state(self) -> Builder:
         return self.with_compilation_state(self.new_compilation_state())
 
-    def with_file_access(self, fa: _data_proxy.FileAccessProvider) -> Builder:
+    def with_file_access(self, fa: FileAccessProvider) -> Builder:
         return self.new_builder().with_file_access(fa)
 
     def with_serialization_settings(self, ss: SerializationSettings) -> Builder:
@@ -481,7 +481,7 @@ class FlyteContext(object):
         in all other cases it is preferable to use with_execution_state
         """
         if not working_dir:
-            working_dir = self.file_access.get_random_local_directory()
+            working_dir = self.file_access.local_sandbox_dir
         return ExecutionState(working_dir=working_dir, user_space_params=self.user_space_params)
 
     @staticmethod
@@ -497,7 +497,7 @@ class FlyteContext(object):
 
     @dataclass
     class Builder(object):
-        file_access: _data_proxy.FileAccessProvider
+        file_access: FileAccessProvider
         level: int = 0
         compilation_state: Optional[CompilationState] = None
         execution_state: Optional[ExecutionState] = None
@@ -550,7 +550,7 @@ class FlyteContext(object):
         def with_new_compilation_state(self) -> "Builder":
             return self.with_compilation_state(self.new_compilation_state())
 
-        def with_file_access(self, fa: _data_proxy.FileAccessProvider) -> "Builder":
+        def with_file_access(self, fa: FileAccessProvider) -> "Builder":
             self.file_access = fa
             return self
 
@@ -642,7 +642,7 @@ class FlyteContextManager(object):
         try:
             yield ctx
         finally:
-            # NOTE: Why? Do we have a loop here to ensure that we are popping all context upto the previously recorded
+            # NOTE: Why? Do we have a loop here to ensure that we are popping all context up to the previously recorded
             # length? This is because it is possible that a conditional context may have leaked. Because of the syntax
             # of conditionals, if a conditional section fails to evaluate / compile, the context is not removed from the
             # stack. This is because context managers cannot be used in the conditional section.
@@ -670,6 +670,11 @@ class FlyteContextManager(object):
         """
         # This is supplied so that tasks that rely on Flyte provided param functionality do not fail when run locally
         default_execution_id = _identifier.WorkflowExecutionIdentifier(project="local", domain="local", name="local")
+
+        # Ensure a local directory is available for users to work with.
+        user_space_path = os.path.join(_sdk_config.LOCAL_SANDBOX.get(), "user_space")
+        pathlib.Path(user_space_path).mkdir(parents=True, exist_ok=True)
+
         # Note we use the SdkWorkflowExecution object purely for formatting into the ex:project:domain:name format users
         # are already acquainted with
         default_user_space_params = ExecutionParameters(
@@ -677,9 +682,9 @@ class FlyteContextManager(object):
             execution_date=_datetime.datetime.utcnow(),
             stats=_mock_stats.MockStats(),
             logging=_logging,
-            tmp_dir=os.path.join(_sdk_config.LOCAL_SANDBOX.get(), "user_space"),
+            tmp_dir=user_space_path,
         )
-        default_context = FlyteContext(file_access=_data_proxy.default_local_file_access_provider)
+        default_context = FlyteContext(file_access=default_local_file_access_provider)
         default_context = default_context.with_execution_state(
             default_context.new_execution_state().with_params(user_space_params=default_user_space_params)
         ).build()
