@@ -9,6 +9,7 @@ from typing import Any, Dict, Generator, List, Optional, Tuple, Type, TypeVar, U
 
 from flytekit.common.exceptions.user import FlyteValidationException
 from flytekit.core import context_manager
+from flytekit.core.docstring import Docstring
 from flytekit.core.type_engine import TypeEngine
 from flytekit.loggers import logger
 from flytekit.models import interface as _interface_models
@@ -24,6 +25,7 @@ class Interface(object):
         inputs: typing.Optional[typing.Dict[str, Union[Type, Tuple[Type, Any]]]] = None,
         outputs: typing.Optional[typing.Dict[str, Type]] = None,
         output_tuple_name: Optional[str] = None,
+        docstring: Optional[Docstring] = None,
     ):
         """
         :param outputs: Output variables and their types as a dictionary
@@ -33,6 +35,7 @@ class Interface(object):
         :param output_tuple_name: This is used to store the name of a typing.NamedTuple when the task or workflow
             returns one. This is also used as a proxy for better or for worse for the presence of a tuple return type,
             primarily used when handling one-element NamedTuples.
+        :param docstring: Docstring of the annotated @task or @workflow from which the interface derives from.
         """
         self._inputs = {}
         if inputs:
@@ -53,7 +56,7 @@ class Interface(object):
                 This class can be used in two different places. For multivariate-return entities this class is used
                 to rewrap the outputs so that our with_overrides function can work.
                 For manual node creation, it's used during local execution as something that can be dereferenced.
-                See the create_node funciton for more information.
+                See the create_node function for more information.
                 """
 
                 def with_overrides(self, *args, **kwargs):
@@ -78,6 +81,7 @@ class Interface(object):
                     ...  # See runs_before
 
             self._output_tuple_class = Output
+        self._docstring = docstring
 
     @property
     def output_tuple(self) -> Optional[Type[collections.namedtuple]]:
@@ -112,6 +116,10 @@ class Interface(object):
     def outputs(self) -> typing.Dict[str, type]:
         return self._outputs
 
+    @property
+    def docstring(self) -> Optional[Docstring]:
+        return self._docstring
+
     def remove_inputs(self, vars: List[str]) -> Interface:
         """
         This method is useful in removing some variables from the Flyte backend inputs specification, as these are
@@ -124,7 +132,7 @@ class Interface(object):
         for v in vars:
             if v in new_inputs:
                 del new_inputs[v]
-        return Interface(new_inputs, self._outputs)
+        return Interface(new_inputs, self._outputs, docstring=self.docstring)
 
     def with_inputs(self, extra_inputs: Dict[str, Type]) -> Interface:
         """
@@ -138,7 +146,7 @@ class Interface(object):
             if k in new_inputs:
                 raise ValueError(f"Input {k} cannot be added as it already exists in the interface")
             new_inputs[k] = v
-        return Interface(new_inputs, self._outputs)
+        return Interface(new_inputs, self._outputs, docstring=self.docstring)
 
     def with_outputs(self, extra_outputs: Dict[str, Type]) -> Interface:
         """
@@ -163,7 +171,10 @@ def transform_inputs_to_parameters(
     """
     if interface is None or interface.inputs_with_defaults is None:
         return _interface_models.ParameterMap({})
-    inputs_vars = transform_variable_map(interface.inputs)
+    if interface.docstring is None:
+        inputs_vars = transform_variable_map(interface.inputs)
+    else:
+        inputs_vars = transform_variable_map(interface.inputs, interface.docstring.input_descriptions)
     params = {}
     inputs_with_def = interface.inputs_with_defaults
     for k, v in inputs_vars.items():
@@ -185,8 +196,16 @@ def transform_interface_to_typed_interface(
     if interface is None:
         return None
 
-    inputs_map = transform_variable_map(interface.inputs)
-    outputs_map = transform_variable_map(interface.outputs)
+    if interface.docstring is None:
+        input_descriptions = output_descriptions = {}
+    else:
+        input_descriptions = interface.docstring.input_descriptions
+        output_descriptions = remap_shared_output_descriptions(
+            interface.docstring.output_descriptions, interface.outputs
+        )
+
+    inputs_map = transform_variable_map(interface.inputs, input_descriptions)
+    outputs_map = transform_variable_map(interface.outputs, output_descriptions)
     return _interface_models.TypedInterface(inputs_map, outputs_map)
 
 
@@ -225,7 +244,7 @@ def transform_interface_to_list_interface(interface: Interface) -> Interface:
     return Interface(inputs=map_inputs, outputs=map_outputs)
 
 
-def transform_signature_to_interface(signature: inspect.Signature) -> Interface:
+def transform_signature_to_interface(signature: inspect.Signature, docstring: Optional[Docstring] = None) -> Interface:
     """
     From the annotations on a task function that the user should have provided, and the output names they want to use
     for each output parameter, construct the TypedInterface object
@@ -250,10 +269,12 @@ def transform_signature_to_interface(signature: inspect.Signature) -> Interface:
             if hasattr(return_annotation, "__name__") and return_annotation.__name__ != "":
                 custom_name = return_annotation.__name__
 
-    return Interface(inputs, outputs, output_tuple_name=custom_name)
+    return Interface(inputs, outputs, output_tuple_name=custom_name, docstring=docstring)
 
 
-def transform_variable_map(variable_map: Dict[str, type]) -> Dict[str, _interface_models.Variable]:
+def transform_variable_map(
+    variable_map: Dict[str, type], descriptions: Dict[str, str] = {}
+) -> Dict[str, _interface_models.Variable]:
     """
     Given a map of str (names of inputs for instance) to their Python native types, return a map of the name to a
     Flyte Variable object with that type.
@@ -261,7 +282,7 @@ def transform_variable_map(variable_map: Dict[str, type]) -> Dict[str, _interfac
     res = OrderedDict()
     if variable_map:
         for k, v in variable_map.items():
-            res[k] = transform_type(v, k)
+            res[k] = transform_type(v, descriptions.get(k, k))
 
     return res
 
@@ -345,3 +366,17 @@ def extract_return_annotation(return_annotation: Union[Type, Tuple]) -> Dict[str
         # Handle all other single return types
         logger.debug(f"Task returns unnamed native tuple {return_annotation}")
         return {default_output_name(): return_annotation}
+
+
+def remap_shared_output_descriptions(output_descriptions: Dict[str, str], outputs: Dict[str, Type]) -> Dict[str, str]:
+    """
+    Deals with mixed styles of return value descriptions used in docstrings. If the docstring contains a single entry of return value description, that output description is shared by each output variable.
+    :param output_descriptions: Dict of output variable names mapping to output description
+    :param outputs: Interface outputs
+    :return: Dict of output variable names mapping to shared output description
+    """
+    # no need to remap
+    if len(output_descriptions) != 1:
+        return output_descriptions
+    _, shared_description = next(iter(output_descriptions.items()))
+    return {k: shared_description for k, _ in outputs.items()}
