@@ -138,7 +138,7 @@ class PythonAutoContainerTask(PythonTask[T], metaclass=FlyteTrackedABC):
     def reset_command_fn(self):
         """
         Resets the command which should be used in the container definition of this task to the default arguments.
-        This is useful when the command line is overriden at serialization time.
+        This is useful when the command line is overridden at serialization time.
         """
         self._get_command_fn = self.get_default_command
 
@@ -158,10 +158,12 @@ class PythonAutoContainerTask(PythonTask[T], metaclass=FlyteTrackedABC):
             data_loading_config=None,
             environment=env,
             storage_request=self.resources.requests.storage,
+            ephemeral_storage_request=self.resources.requests.ephemeral_storage,
             cpu_request=self.resources.requests.cpu,
             gpu_request=self.resources.requests.gpu,
             memory_request=self.resources.requests.mem,
             storage_limit=self.resources.limits.storage,
+            ephemeral_storage_limit=self.resources.limits.ephemeral_storage,
             cpu_limit=self.resources.limits.cpu,
             gpu_limit=self.resources.limits.gpu,
             memory_limit=self.resources.limits.mem,
