@@ -3,9 +3,9 @@ from __future__ import annotations
 import dataclasses
 import datetime as _datetime
 import enum
+import inspect
 import json as _json
 import mimetypes
-import os
 import typing
 from abc import ABC, abstractmethod
 from typing import Type
@@ -21,11 +21,12 @@ from marshmallow_jsonschema import JSONSchema
 
 from flytekit.common.types import primitives as _primitives
 from flytekit.core.context_manager import FlyteContext
+from flytekit.core.type_helpers import load_type_from_tag
 from flytekit.loggers import logger
 from flytekit.models import interface as _interface_models
 from flytekit.models import types as _type_models
 from flytekit.models.core import types as _core_types
-from flytekit.models.literals import Blob, BlobMetadata, Literal, LiteralCollection, LiteralMap, Primitive, Scalar
+from flytekit.models.literals import Literal, LiteralCollection, LiteralMap, Primitive, Scalar
 from flytekit.models.types import LiteralType, SimpleType
 
 T = typing.TypeVar("T")
@@ -270,14 +271,24 @@ class ProtobufTransformer(TypeTransformer[_proto_reflection.GeneratedProtocolMes
         return Literal(scalar=Scalar(generic=struct))
 
     def to_python_value(self, ctx: FlyteContext, lv: Literal, expected_python_type: Type[T]) -> T:
-        if not (lv and lv.scalar and lv.scalar.generic):
-            raise AssertionError("Can only covert a generic literal to a Protobuf")
+        if not (lv and lv.scalar and lv.scalar.generic is not None):
+            raise AssertionError("Can only convert a generic literal to a Protobuf")
 
         pb_obj = expected_python_type()
         dictionary = _MessageToDict(lv.scalar.generic)
         pb_obj = _ParseDict(dictionary, pb_obj)
         return pb_obj
 
+    def guess_python_type(self, literal_type: LiteralType) -> Type[T]:
+        if (
+            literal_type.simple == SimpleType.STRUCT
+            and literal_type.metadata
+            and literal_type.metadata.get(self.PB_FIELD_KEY, "")
+        ):
+            tag = literal_type.metadata[self.PB_FIELD_KEY]
+            return load_type_from_tag(tag)
+        raise ValueError(f"Transformer {self} cannot reverse {literal_type}")
+
 
 class TypeEngine(typing.Generic[T]):
     """
@@ -290,17 +301,19 @@ class TypeEngine(typing.Generic[T]):
     _DATACLASS_TRANSFORMER: TypeTransformer = DataclassTransformer()
 
     @classmethod
-    def register(cls, transformer: TypeTransformer):
+    def register(cls, transformer: TypeTransformer, additional_types: typing.Optional[typing.List[Type]] = None):
         """
         This should be used for all types that respond with the right type annotation when you use type(...) function
         """
-        if transformer.python_type in cls._REGISTRY:
-            existing = cls._REGISTRY[transformer.python_type]
-            raise ValueError(
-                f"Transformer {existing.name} for type {transformer.python_type} is already registered."
-                f" Cannot override with {transformer.name}"
-            )
-        cls._REGISTRY[transformer.python_type] = transformer
+        types = [transformer.python_type, *(additional_types or [])]
+        for t in types:
+            if t in cls._REGISTRY:
+                existing = cls._REGISTRY[t]
+                raise ValueError(
+                    f"Transformer {existing.name} for type {t} is already registered."
+                    f" Cannot override with {transformer.name}"
+                )
+            cls._REGISTRY[t] = transformer
 
     @classmethod
     def get_transformer(cls, python_type: Type) -> TypeTransformer[T]:
@@ -320,7 +333,7 @@ class TypeEngine(typing.Generic[T]):
             if v is of type data class, use the dataclass transformer
 
         Step 4:
-            Walk the inheritance hierarchy of v and  find a transformer that matches the first base class.
+            Walk the inheritance hierarchy of v and find a transformer that matches the first base class.
             This is potentially non-deterministic - will depend on the registration pattern.
 
             TODO lets make this deterministic by using an ordered dict
@@ -345,7 +358,9 @@ class TypeEngine(typing.Generic[T]):
         for base_type in cls._REGISTRY.keys():
             if base_type is None:
                 continue  # None is actually one of the keys, but isinstance/issubclass doesn't work on it
-            if isinstance(python_type, base_type) or issubclass(python_type, base_type):
+            if isinstance(python_type, base_type) or (
+                inspect.isclass(python_type) and issubclass(python_type, base_type)
+            ):
                 return cls._REGISTRY[base_type]
         raise ValueError(f"Type {python_type} not supported currently in Flytekit. Please register a new transformer")
 
@@ -403,17 +418,28 @@ class TypeEngine(typing.Generic[T]):
         return {k: TypeEngine.to_python_value(ctx, lm.literals[k], v) for k, v in python_types.items()}
 
     @classmethod
-    def dict_to_literal_map(cls, ctx: FlyteContext, d: typing.Dict[str, typing.Any]) -> LiteralMap:
+    def dict_to_literal_map(
+        cls,
+        ctx: FlyteContext,
+        d: typing.Dict[str, typing.Any],
+        guessed_python_types: typing.Optional[typing.Dict[str, type]] = None,
+    ) -> LiteralMap:
         """
-        Given a dictionary mapping string keys to python values, convert to a LiteralMap.
+        Given a dictionary mapping string keys to python values and a dictionary containing guessed types for such string keys,
+        convert to a LiteralMap.
         """
+        guessed_python_types = guessed_python_types or {}
         literal_map = {}
         for k, v in d.items():
+            # The guessed type takes precedence over the type returned by the python runtime. This is needed
+            # to account for the type erasure that happens in the case of built-in collection containers, such as
+            # `list` and `dict`.
+            python_type = guessed_python_types.get(k, type(v))
             literal_map[k] = TypeEngine.to_literal(
                 ctx=ctx,
                 python_val=v,
-                python_type=type(v),
-                expected=TypeEngine.to_literal_type(type(v)),
+                python_type=python_type,
+                expected=TypeEngine.to_literal_type(python_type),
             )
         return LiteralMap(literal_map)
 
@@ -643,55 +669,6 @@ class BinaryIOTransformer(TypeTransformer[typing.BinaryIO]):
         return open(local_path, "rb")
 
 
-class PathLikeTransformer(TypeTransformer[os.PathLike]):
-    """
-    Handler for os.PathLike
-    """
-
-    def __init__(self):
-        super().__init__(name="os.PathLike", t=os.PathLike)
-
-    def _blob_type(self) -> _core_types.BlobType:
-        return _core_types.BlobType(
-            format=mimetypes.types_map[".bin"],
-            dimensionality=_core_types.BlobType.BlobDimensionality.SINGLE,
-        )
-
-    def get_literal_type(self, t: Type[os.PathLike]) -> LiteralType:
-        return _type_models.LiteralType(
-            blob=self._blob_type(),
-        )
-
-    def to_literal(
-        self, ctx: FlyteContext, python_val: os.PathLike, python_type: Type[os.PathLike], expected: LiteralType
-    ) -> Literal:
-        # TODO we could guess the mimetype and allow the format to be changed at runtime. thus a non existent format
-        #      could be replaced with a guess format?
-
-        rpath = ctx.file_access.get_random_remote_path()
-
-        # For remote values, say https://raw.github.com/demo_data.csv, we will not upload to Flyte's store (S3/GCS)
-        # and just return a literal with a uri equal to the path given
-        if ctx.file_access.is_remote(python_val):
-            return Literal(scalar=Scalar(blob=Blob(metadata=BlobMetadata(expected.blob), uri=python_val)))
-
-        # For local files, we'll upload for the user.
-        ctx.file_access.put_data(python_val, rpath, is_multipart=False)
-        return Literal(scalar=Scalar(blob=Blob(metadata=BlobMetadata(expected.blob), uri=rpath)))
-
-    def to_python_value(self, ctx: FlyteContext, lv: Literal, expected_python_type: Type[os.PathLike]) -> os.PathLike:
-        # TODO rename to get_auto_local_path()
-        local_destination_path = ctx.file_access.get_random_local_path()
-        uri = lv.scalar.blob.uri
-        # If the uri is just a local path like /tmp/file_name, we just return
-        if not ctx.file_access.is_remote(uri):
-            return uri
-
-        # Since no delayed downloading is possible with strings, always download immediately.
-        ctx.file_access.get_data(lv.scalar.blob.uri, local_destination_path, is_multipart=False)
-        return local_destination_path
-
-
 class EnumTransformer(TypeTransformer[enum.Enum]):
     """
     Enables converting a python type enum.Enum to LiteralType.EnumType
@@ -794,9 +771,9 @@ def _register_default_type_transformers():
     TypeEngine.register(ListTransformer())
     TypeEngine.register(DictTransformer())
     TypeEngine.register(TextIOTransformer())
-    TypeEngine.register(PathLikeTransformer())
     TypeEngine.register(BinaryIOTransformer())
     TypeEngine.register(EnumTransformer())
+    TypeEngine.register(ProtobufTransformer())
 
     # inner type is. Also unsupported are typing's Tuples. Even though you can look inside them, Flyte's type system
     # doesn't support these currently.
@@ -810,5 +787,3 @@ def _register_default_type_transformers():
 
 
 _register_default_type_transformers()
-
-TypeEngine.register(ProtobufTransformer())
