@@ -1,11 +1,11 @@
-from __future__ import absolute_import
+class DataProxy(object):
+    def __init__(self, name: str):
+        self._name = name
 
-import abc as _abc
+    @property
+    def name(self) -> str:
+        return self._name
 
-import six as _six
-
-
-class DataProxy(_six.with_metaclass(_abc.ABCMeta, object)):
     def exists(self, path):
         """
         :param path:
