@@ -1,7 +1,3 @@
-from __future__ import absolute_import
-
-import six as _six
-
 from flytekit.common import constants as _constants
 from flytekit.common import utils as _common_utils
 from flytekit.common.exceptions import user as _user_exception
@@ -14,7 +10,6 @@ from flytekit.interfaces.data.s3 import s3proxy as _s3proxy
 
 
 class LocalWorkingDirectoryContext(object):
-
     _CONTEXTS = []
 
     def __init__(self, directory):
@@ -32,7 +27,6 @@ class LocalWorkingDirectoryContext(object):
 
 
 class _OutputDataContext(object):
-
     _CONTEXTS = [_local_file_proxy.LocalFileProxy(_sdk_config.LOCAL_SANDBOX.get())]
 
     def __init__(self, context):
@@ -62,7 +56,6 @@ class LocalDataContext(_OutputDataContext):
 
 
 class RemoteDataContext(_OutputDataContext):
-
     _CLOUD_PROVIDER_TO_PROXIES = {
         _constants.CloudProvider.AWS: _s3proxy.AwsS3Proxy,
         _constants.CloudProvider.GCP: _gcs_proxy.GCSProxy,
@@ -99,7 +92,7 @@ class Data(object):
         :param Text path:
         :rtype: flytekit.interfaces.data.common.DataProxy
         """
-        for k, v in _six.iteritems(cls._DATA_PROXIES):
+        for k, v in cls._DATA_PROXIES.items():
             if path.startswith(k):
                 return v
         return _OutputDataContext.get_default_proxy()
@@ -135,7 +128,7 @@ class Data(object):
                     remote_path=remote_path,
                     local_path=local_path,
                     is_multipart=is_multipart,
-                    error_string=_six.text_type(ex),
+                    error_string=str(ex),
                 )
             )
 
@@ -160,7 +153,7 @@ class Data(object):
                     remote_path=remote_path,
                     local_path=local_path,
                     is_multipart=is_multipart,
-                    error_string=_six.text_type(ex),
+                    error_string=str(ex),
                 )
             )
 
