@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import requests as _requests
 
 from flytekit.common.exceptions import user as _user_exceptions
@@ -12,6 +10,9 @@ class HttpFileProxy(_common_data.DataProxy):
     _HTTP_FORBIDDEN = 403
     _HTTP_NOT_FOUND = 404
 
+    def __init__(self):
+        super(HttpFileProxy, self).__init__(name="http")
+
     def exists(self, path):
         """
         :param Text path: the path of the file
