@@ -1,10 +1,11 @@
-from __future__ import absolute_import
-
+import logging
 import os as _os
 import re as _re
 import string as _string
 import sys as _sys
+import time
 import uuid as _uuid
+from typing import Dict, List
 
 from six import moves as _six_moves
 from six import text_type as _text_type
@@ -21,9 +22,12 @@ else:
     from distutils.spawn import find_executable as _which
 
 
-def _update_cmd_config_and_execute(cmd):
+def _update_cmd_config_and_execute(cmd: List[str]):
     env = _os.environ.copy()
 
+    if _aws_config.ENABLE_DEBUG.get():
+        cmd.insert(1, "--debug")
+
     if _aws_config.S3_ENDPOINT.get() is not None:
         cmd.insert(1, _aws_config.S3_ENDPOINT.get())
         cmd.insert(1, _aws_config.S3_ENDPOINT_ARG_NAME)
@@ -34,7 +38,30 @@ def _update_cmd_config_and_execute(cmd):
     if _aws_config.S3_SECRET_ACCESS_KEY.get() is not None:
         env[_aws_config.S3_SECRET_ACCESS_KEY_ENV_NAME] = _aws_config.S3_SECRET_ACCESS_KEY.get()
 
-    return _subprocess.check_call(cmd, env=env)
+    retry = 0
+    while True:
+        try:
+            return _subprocess.check_call(cmd, env=env)
+        except Exception as e:
+            logging.error(f"Exception when trying to execute {cmd}, reason: {str(e)}")
+            retry += 1
+            if retry > _aws_config.RETRIES.get():
+                raise
+            secs = _aws_config.BACKOFF_SECONDS.get()
+            logging.info(f"Sleeping before retrying again, after {secs} seconds")
+            time.sleep(secs)
+            logging.info("Retrying again")
+
+
+def _extra_args(extra_args: Dict[str, str]) -> List[str]:
+    cmd = []
+    if "ContentType" in extra_args:
+        cmd += ["--content-type", extra_args["ContentType"]]
+    if "ContentEncoding" in extra_args:
+        cmd += ["--content-encoding", extra_args["ContentEncoding"]]
+    if "ACL" in extra_args:
+        cmd += ["--acl", extra_args["ACL"]]
+    return cmd
 
 
 class AwsS3Proxy(_common_data.DataProxy):
@@ -48,6 +75,7 @@ class AwsS3Proxy(_common_data.DataProxy):
             path (_get_shard_path), use this prefix instead as a base. This code assumes that the
             path passed in is correct. That is, an S3 path won't be passed in when running on GCP.
         """
+        super().__init__(name="awscli-s3")
         self._raw_output_data_prefix_override = raw_output_data_prefix_override
 
     @property
@@ -142,12 +170,7 @@ class AwsS3Proxy(_common_data.DataProxy):
         }
 
         cmd = [AwsS3Proxy._AWS_CLI, "s3", "cp"]
-        if "ContentType" in extra_args:
-            cmd += ["--content-type", extra_args["ContentType"]]
-        if "ContentEncoding" in extra_args:
-            cmd += ["--content-encoding", extra_args["ContentEncoding"]]
-        if "ACL" in extra_args:
-            cmd += ["--acl", extra_args["ACL"]]
+        cmd.extend(_extra_args(extra_args))
         cmd += [file_path, to_path]
 
         return _update_cmd_config_and_execute(cmd)
@@ -166,12 +189,7 @@ class AwsS3Proxy(_common_data.DataProxy):
 
         AwsS3Proxy._check_binary()
         cmd = [AwsS3Proxy._AWS_CLI, "s3", "cp", "--recursive"]
-        if "ContentType" in extra_args:
-            cmd += ["--content-type", extra_args["ContentType"]]
-        if "ContentEncoding" in extra_args:
-            cmd += ["--content-encoding", extra_args["ContentEncoding"]]
-        if "ACL" in extra_args:
-            cmd += ["--acl", extra_args["ACL"]]
+        cmd.extend(_extra_args(extra_args))
         cmd += [local_path, remote_path]
         return _update_cmd_config_and_execute(cmd)
 
