@@ -1,14 +1,16 @@
-from __future__ import absolute_import
+import datetime
+import typing
 
 from flyteidl.core import workflow_pb2 as _core_workflow
 
 from flytekit.models import common as _common
 from flytekit.models import interface as _interface
+from flytekit.models import types as _types
 from flytekit.models.core import condition as _condition
-from flytekit.models.core import errors as _errors
 from flytekit.models.core import identifier as _identifier
 from flytekit.models.literals import Binding as _Binding
 from flytekit.models.literals import RetryStrategy as _RetryStrategy
+from flytekit.models.task import Resources
 
 
 class IfBlock(_common.FlyteIdlEntity):
@@ -40,7 +42,7 @@ class IfBlock(_common.FlyteIdlEntity):
         """
         :rtype: flyteidl.core.workflow_pb2.IfBlock
         """
-        return _core_workflow.IfBlock(condition=self.condition.to_flyte_idl(), then_node=self.then_node.to_flyte_idl(),)
+        return _core_workflow.IfBlock(condition=self.condition.to_flyte_idl(), then_node=self.then_node.to_flyte_idl())
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -59,9 +61,8 @@ class IfElseBlock(_common.FlyteIdlEntity):
         :param IfBlock case:
         :param list[IfBlock] other:
         :param Node else_node:
-        :param flytekit.models.core.errors.ContainerError error:
+        :param _types.Error error:
         """
-
         self._case = case
         self._other = other
         self._else_node = else_node
@@ -120,12 +121,12 @@ class IfElseBlock(_common.FlyteIdlEntity):
             case=IfBlock.from_flyte_idl(pb2_object.case),
             other=[IfBlock.from_flyte_idl(a) for a in pb2_object.other],
             else_node=Node.from_flyte_idl(pb2_object.else_node) if pb2_object.HasField("else_node") else None,
-            error=_errors.ContainerError.from_flyte_idl(pb2_object.error) if pb2_object.HasField("error") else None,
+            error=_types.Error.from_flyte_idl(pb2_object.error) if pb2_object.HasField("error") else None,
         )
 
 
 class BranchNode(_common.FlyteIdlEntity):
-    def __init__(self, if_else):
+    def __init__(self, if_else: IfElseBlock):
         """
         BranchNode is a special node that alter the flow of the workflow graph. It allows the control flow to branch at
         runtime based on a series of conditions that get evaluated on various parameters (e.g. inputs, primtives).
@@ -135,7 +136,7 @@ class BranchNode(_common.FlyteIdlEntity):
         self._if_else = if_else
 
     @property
-    def if_else(self):
+    def if_else(self) -> IfElseBlock:
         """
         :rtype: IfElseBlock
         """
@@ -154,17 +155,18 @@ class BranchNode(_common.FlyteIdlEntity):
 
 
 class NodeMetadata(_common.FlyteIdlEntity):
-    def __init__(self, name, timeout, retries, interruptible=False):
+    def __init__(self, name, timeout=None, retries=None, interruptible=None):
         """
         Defines extra information about the Node.
 
         :param Text name: Friendly name for the Node.
-        :param datetime.timedelta timeout: Overall timeout for a task.
-        :param flytekit.models.literals.RetryStrategy retries: Number of retries per task.
+        :param datetime.timedelta timeout: [Optional] Overall timeout for a task.
+        :param flytekit.models.literals.RetryStrategy retries: [Optional] Number of retries per task.
+        :param bool interruptible: [Optional] Can be safely interrupted during execution.
         """
         self._name = name
-        self._timeout = timeout
-        self._retries = retries
+        self._timeout = timeout if timeout is not None else datetime.timedelta()
+        self._retries = retries if retries is not None else _RetryStrategy(0)
         self._interruptible = interruptible
 
     @property
@@ -200,15 +202,20 @@ class NodeMetadata(_common.FlyteIdlEntity):
         :rtype: flyteidl.core.workflow_pb2.NodeMetadata
         """
         node_metadata = _core_workflow.NodeMetadata(
-            name=self.name, retries=self.retries.to_flyte_idl(), interruptible=self.interruptible,
+            name=self.name,
+            retries=self.retries.to_flyte_idl(),
+            interruptible=self.interruptible,
         )
-        node_metadata.timeout.FromTimedelta(self.timeout)
+        if self.timeout:
+            node_metadata.timeout.FromTimedelta(self.timeout)
         return node_metadata
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
         return cls(
-            pb2_object.name, pb2_object.timeout.ToTimedelta(), _RetryStrategy.from_flyte_idl(pb2_object.retries),
+            pb2_object.name,
+            pb2_object.timeout.ToTimedelta(),
+            _RetryStrategy.from_flyte_idl(pb2_object.retries),
         )
 
 
@@ -233,7 +240,7 @@ class Node(_common.FlyteIdlEntity):
         :param NodeMetadata metadata: Extra metadata about the node.
         :param list[flytekit.models.literals.Binding] inputs: Specifies how to bind the underlying
             interface's inputs.  All required inputs specified in the underlying interface must be fulfilled.
-        :param list[Text] upstream_node_ids: Specifies execution depdendency for this node ensuring it will
+        :param list[Text] upstream_node_ids: Specifies execution dependency for this node ensuring it will
             only get scheduled to run after all its upstream nodes have completed. This node will have
             an implicit dependency on any node that appears in inputs field.
         :param list[Alias] output_aliases: A node can define aliases for a subset of its outputs. This
@@ -248,6 +255,7 @@ class Node(_common.FlyteIdlEntity):
         self._metadata = metadata
         self._inputs = inputs
         self._upstream_node_ids = upstream_node_ids
+        # TODO: For proper graph handling, we need to keep track of the node objects themselves, not just the node IDs
         self._output_aliases = output_aliases
         self._task_node = task_node
         self._workflow_node = workflow_node
@@ -282,7 +290,7 @@ class Node(_common.FlyteIdlEntity):
     @property
     def upstream_node_ids(self):
         """
-        [Optional] Specifies execution depdendency for this node ensuring it will
+        [Optional] Specifies execution dependency for this node ensuring it will
         only get scheduled to run after all its upstream nodes have completed. This node will have
         an implicit dependency on any node that appears in inputs field.
         :rtype: list[Text]
@@ -367,16 +375,39 @@ class Node(_common.FlyteIdlEntity):
         )
 
 
+class TaskNodeOverrides(_common.FlyteIdlEntity):
+    def __init__(self, resources: typing.Optional[Resources] = None):
+        self._resources = resources
+
+    @property
+    def resources(self) -> Resources:
+        return self._resources
+
+    def to_flyte_idl(self):
+        return _core_workflow.TaskNodeOverrides(
+            resources=self.resources.to_flyte_idl() if self.resources is not None else None,
+        )
+
+    @classmethod
+    def from_flyte_idl(cls, pb2_object):
+        resources = Resources.from_flyte_idl(pb2_object.resources)
+        if bool(resources.requests) or bool(resources.limits):
+            return cls(resources=resources)
+        return cls(resources=None)
+
+
 class TaskNode(_common.FlyteIdlEntity):
-    def __init__(self, reference_id):
+    def __init__(self, reference_id, overrides: typing.Optional[TaskNodeOverrides] = None):
         """
         Refers to the task that the Node is to execute.
         NB: This is currently a oneof in protobuf, but there's only one option currently.  This code should be updated
             when more options are available.
 
         :param flytekit.models.core.identifier.Identifier reference_id: A globally unique identifier for the task.
+        :param flyteidl.core.workflow_pb2.TaskNodeOverrides
         """
         self._reference_id = reference_id
+        self._overrides = overrides
 
     @property
     def reference_id(self):
@@ -386,11 +417,18 @@ class TaskNode(_common.FlyteIdlEntity):
         """
         return self._reference_id
 
+    @property
+    def overrides(self) -> TaskNodeOverrides:
+        return self._overrides
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.core.workflow_pb2.TaskNode
         """
-        return _core_workflow.TaskNode(reference_id=self.reference_id.to_flyte_idl())
+        return _core_workflow.TaskNode(
+            reference_id=self.reference_id.to_flyte_idl(),
+            overrides=self.overrides.to_flyte_idl() if self.overrides is not None else None,
+        )
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -398,7 +436,13 @@ class TaskNode(_common.FlyteIdlEntity):
         :param flyteidl.core.workflow_pb2.TaskNode pb2_object:
         :rtype: TaskNode
         """
-        return cls(reference_id=_identifier.Identifier.from_flyte_idl(pb2_object.reference_id))
+        overrides = TaskNodeOverrides.from_flyte_idl(pb2_object.overrides)
+        if overrides.resources is None:
+            overrides = None
+        return cls(
+            reference_id=_identifier.Identifier.from_flyte_idl(pb2_object.reference_id),
+            overrides=overrides,
+        )
 
 
 class WorkflowNode(_common.FlyteIdlEntity):
@@ -481,7 +525,6 @@ class WorkflowMetadata(_common.FlyteIdlEntity):
     def __init__(self, on_failure=None):
         """
         Metadata for the workflow.
-
         :param on_failure flytekit.models.core.workflow.WorkflowMetadata.OnFailurePolicy: [Optional] The execution policy when the workflow detects a failure.
         """
         self._on_failure = on_failure
@@ -520,13 +563,17 @@ class WorkflowMetadataDefaults(_common.FlyteIdlEntity):
         """
         Metadata Defaults for the workflow.
         """
-        self.interruptible_ = interruptible
+        self._interruptible = interruptible
+
+    @property
+    def interruptible(self):
+        return self._interruptible
 
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.core.workflow_pb2.WorkflowMetadataDefaults
         """
-        return _core_workflow.WorkflowMetadataDefaults(interruptible=self.interruptible_)
+        return _core_workflow.WorkflowMetadataDefaults(interruptible=self._interruptible)
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -539,7 +586,14 @@ class WorkflowMetadataDefaults(_common.FlyteIdlEntity):
 
 class WorkflowTemplate(_common.FlyteIdlEntity):
     def __init__(
-        self, id, metadata, metadata_defaults, interface, nodes, outputs, failure_node=None,
+        self,
+        id,
+        metadata,
+        metadata_defaults,
+        interface,
+        nodes,
+        outputs,
+        failure_node=None,
     ):
         """
         A workflow template encapsulates all the task, branch, and subworkflow nodes to run a statically analyzable,
