@@ -1,6 +1,4 @@
-from __future__ import absolute_import
-
-## Todo - change this to qubole_presto once Luis's PR get's merged
+## Todo - change this to qubole_presto once Luis's PR gets merged
 # from flyteidl.plugins import qubole_presto as _qubole
 from flyteidl.plugins import presto_pb2 as _presto
 
@@ -8,7 +6,7 @@ from flytekit.models import common as _common
 
 
 class PrestoQuery(_common.FlyteIdlEntity):
-    def __init__(self, routing_group, catalog, schema, statement):
+    def __init__(self, routing_group=None, catalog=None, schema=None, statement=None):
         """
         Initializes a new PrestoQuery.
 
@@ -57,7 +55,10 @@ class PrestoQuery(_common.FlyteIdlEntity):
         :rtype: _presto.PrestoQuery
         """
         return _presto.PrestoQuery(
-            routing_group=self._routing_group, catalog=self._catalog, schema=self._schema, statement=self._statement,
+            routing_group=self._routing_group,
+            catalog=self._catalog,
+            schema=self._schema,
+            statement=self._statement,
         )
 
     @classmethod
